﻿#r "../packages/FSharp.Data.2.4.6/lib/net45/FSharp.Data.dll"
#r "../packages/Accord.3.8.0/lib/net40/Accord.dll"
#r "../packages/Accord.Statistics.3.8.0/lib/net40/Accord.Statistics.dll"
#r "../packages/Accord.Math.3.8.0/lib/net40/Accord.Math.dll"
#r "../packages/Accord.Math.3.8.0/lib/net40/Accord.Math.Core.dll"

open Accord.Statistics.Models.Regression.Linear
open FSharp.Data

type CupcakeSale = 
    {CupcakeId : int;
     Type : string;
     Price : float;
     CustomerId : int;
     Rating : float;
     Total: float}

let sales = ResizeArray<CupcakeSale>()

type CupcakeSalesCSV = CsvProvider<"cupcake_sales.csv", ",">
let csv = new CupcakeSalesCSV()

for row in csv.Rows do
    sales.Add({CupcakeId = row.CupcakeId;
        Type = row.Type;
        Price = (float) row.Price;
        CustomerId = row.CustomerId;
        Rating = (float) row.Rating;
        Total = (float) row.Total})

let filterByRating = [(fun (x : CupcakeSale) -> x.Rating)]
let filterByTotal = [(fun (x : CupcakeSale) -> x.Total)]

let groupedByRatings = 
    sales
    |> Seq.groupBy(fun x -> x.CupcakeId)
    |> Seq.map(fun (key, values) -> 
           (key, [for prop in filterByRating -> values |> Seq.averageBy prop]))
    |> Seq.toArray

let groupedByTotals = 
    sales
    |> Seq.groupBy(fun x -> x.CupcakeId)
    |> Seq.map(fun (key, values) -> 
           (key, [for prop in filterByTotal -> values |> Seq.averageBy prop]))
    |> Seq.toArray

let cupcakeIds = Array.create groupedByRatings.Length 0
let x_averages = Array.create groupedByRatings.Length 0.0
let y_totals = Array.create groupedByRatings.Length 0.0

for i in 0..groupedByRatings.Length - 1 do
    let averageTuple = Array.get groupedByRatings i
    let totalTuple = Array.get groupedByTotals i

    let _id = fst averageTuple
    let _average = (snd averageTuple).[0]
    let _total = (snd totalTuple).[0]

    Array.set cupcakeIds i _id
    Array.set x_averages i _average
    Array.set y_totals i _total

let ols = OrdinaryLeastSquares()
let regression = ols.Learn(x_averages, y_totals)

// Let's say you received this value from the form
let purchasedCupcakeId = 1;

let findIndex arr elem = arr |> Array.findIndex((=) elem)
let i = findIndex cupcakeIds purchasedCupcakeId

let result = regression.Transform(Array.get x_averages i)
System.Console.WriteLine("Suggested quantity: {0}", System.Math.Round(result))